revparse: return trees through the "colon" syntax
1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60
diff --git a/tests-clar/refs/revparse.c b/tests-clar/refs/revparse.c
index a468a4c..b8b1ed9 100644
--- a/tests-clar/refs/revparse.c
+++ b/tests-clar/refs/revparse.c
@@ -166,8 +166,13 @@ void test_refs_revparse__colon(void)
cl_git_fail(git_revparse_single(&g_obj, g_repo, ":/"));
cl_git_fail(git_revparse_single(&g_obj, g_repo, ":/not found in any commit"));
cl_git_fail(git_revparse_single(&g_obj, g_repo, ":2:README"));
- cl_git_fail(git_revparse_single(&g_obj, g_repo, "master:"));
+ /* Trees */
+ test_object("master:", "944c0f6e4dfa41595e6eb3ceecdb14f50fe18162");
+ test_object("subtrees:", "ae90f12eea699729ed24555e40b9fd669da12a12");
+ test_object("subtrees:ab", "f1425cef211cc08caa31e7b545ffb232acb098c3");
+
+ /* Blobs */
test_object("subtrees:ab/4.txt", "d6c93164c249c8000205dd4ec5cbca1b516d487f");
test_object("subtrees:ab/de/fgh/1.txt", "1f67fc4386b2d171e0d21be1c447e12660561f9b");
test_object("master:README", "a8233120f6ad708f843d861ce2b7228ec4e3dec6");
diff --git a/src/revparse.c b/src/revparse.c
index bc750f4..4bc6849 100644
--- a/src/revparse.c
+++ b/src/revparse.c
@@ -533,16 +533,24 @@ static int handle_linear_syntax(git_object **out, git_object *obj, const char *m
static int oid_for_tree_path(git_oid *out, git_tree *tree, git_repository *repo, const char *path)
{
- char *str = git__strdup(path);
- char *tok;
- void *alloc = str;
+ char *str, *tok;
+ void *alloc;
git_tree *tree2 = tree;
const git_tree_entry *entry = NULL;
+ if (*path == '\0') {
+ git_oid_cpy(out, git_object_id((git_object *)tree));
+ return 0;
+ }
+
+ alloc = str = git__strdup(path);
+
while ((tok = git__strtok(&str, "/\\")) != NULL) {
entry = git_tree_entry_byname(tree2, tok);
if (tree2 != tree) git_tree_free(tree2);
if (git_tree_entry__is_tree(entry)) {
+ if (str == '\0')
+ break;
if (git_tree_lookup(&tree2, repo, &entry->oid) < 0) {
git__free(alloc);
return GIT_ERROR;
@@ -576,7 +584,7 @@ static int handle_colon_syntax(git_object **out,
}
tree = (git_tree*)obj;
/* Find the blob at the given path. */
error = oid_for_tree_path(&oid, tree, repo, path);
git_tree_free(tree);
@@ -576,7 +584,7 @@ static int handle_colon_syntax(git_object **out,
}
tree = (git_tree*)obj;
- /* Find the blob at the given path. */
+ /* Find the blob or tree at the given path. */
error = oid_for_tree_path(&oid, tree, repo, path);
git_tree_free(tree);